// SLDWORKS_EXTRACTBITMAPIMPL : Implementation of TSldWorks_ExtractBitmapImpl (CoClass: SldWorks_ExtractBitmap, Interface: ISldWorks_ExtractBitmap)

#include <vcl.h>
#pragma hdrstop

#include <assert.h>
#include "SLDWORKS_EXTRACTBITMAPIMPL.H"

/////////////////////////////////////////////////////////////////////////////
// TSldWorks_ExtractBitmapImpl

// stolen from swconst.h
enum swFileLoadError_e
{
    swGenericError                  = 0x1,
	swFileNotFoundError             = 0x2,
	swIdMatchError                  = 0x4,
	swReadOnlyWarn                  = 0x8,
	swSharingViolationWarn          = 0x10,
	swDrawingANSIUpdateWarn         = 0x20,
	swSheetScaleUpdateWarn          = 0x40,
	swNeedsRegenWarn                = 0x80,
	swBasePartNotLoadedWarn         = 0x100,
	swFileAlreadyOpenWarn           = 0x200,    // the requested file is already open, that document will be used
	swInvalidFileTypeError          = 0x400,	// the type argument passed into the API is not valid
	swDrawingsOnlyRapidDraftWarn    = 0x800,	// only drawings can be converted to RapidDraft format
	swViewOnlyRestrictions          = 0x1000,	// a document being opened view only can not be configured
	swFutureVersion                 = 0x2000,	// document being opened is of a future version.
	swViewMissingReferencedConfig   = 0x4000,	// drawing view referenced config is missing frpm model
	swDrawingSFSymbolConvertWarn    = 0x8000	// drawing contains old style surface finish symbols
};
//-----------------------------------------------------------------------
STDMETHODIMP TSldWorks_ExtractBitmapImpl::ExtractBitmap
(
    BSTR                    SolidWorksFilename,
    BSTR                    BitmapFilename,
    long                    Width,
    long                    Height,
    long*                   Error
)
// parameters
//      [in]                SolidWorksFilename
//          path to SolidWorks file, including file extension
//          path should really be fully qualified
//
//      [in]                BitmapFilename
//          path to output bitmap, including file extension
//          a default file extension will NOT be added 
//          this file will be overwritten if it exists
//          path should really be fully qualified
//
//      [in]                Width
//          width of output bitmap in pixels
//
//      [in]                Height
//          height of output bitmap in pixels
//
//      [out, retval]       Error
//          code from swFileLoadError_e - either swGenericError or swFileNotFoundError
//          zero if successful
//
// returns
//      S_OK
//          if file successfully saved
//
//      S_FALSE
//          could not open top level storage or bitmap stream in SolidWorks file
//          or could not delete existing bitmap file
//
//      E_POINTER
//          NULL pointer passed for error parameter
//
//      E_INVALIDARG
//          width or height less than or equal to zero
//
// notes
//
{
    ObjectLock(this);

    try
        {
        const int                       Offset = 3;    // to have the frame look good

        CComPtr <IStorage>              pRoot;
        CComPtr <IStream>               pPreview;
        DWORD                           dwDataSize = -1;
        DWORD                           dwNumRead  = -1;
        char*                           pDIBData = NULL;
        LPBITMAPINFOHEADER              lpbihDIBData = NULL;
        int                             NumLines = -1;
        Graphics::TBitmap*              SWImage = NULL;
        HRESULT                         hr = S_OK;

        if (NULL == Error)
            {
            return E_POINTER;
            }
        *Error = swGenericError;

        if (Width <= 0 || Height <= 0)
            {
            return E_INVALIDARG;
            }


        // first check the SW file is there
        if (!FileExists(SolidWorksFilename))
            {
            *Error = swFileNotFoundError;
            }

        // open up SW file
        hr = StgOpenStorage(WideString(SolidWorksFilename), NULL,
                        STGM_DIRECT | STGM_READ | STGM_SHARE_EXCLUSIVE,
                        NULL, 0, &pRoot);
        if (NULL == pRoot)
            {
            // something went wrong, probably a sharing violation
            return S_FALSE;
            }

        // open up bitmap preview stream
        hr = pRoot->OpenStream(WideString("Preview"),NULL,
                    STGM_DIRECT | STGM_READ | STGM_SHARE_EXCLUSIVE,
                    0, &pPreview);
        if (NULL == pPreview)
            {
            // something went wrong, probably no preview
            return S_FALSE;
            }

        // read bitmap from stream
        hr = pPreview->Read(&dwDataSize, sizeof(DWORD), &dwNumRead);
        if (dwDataSize > 0)
            {
            // delete the existing bitmap file
            if (FileExists(BitmapFilename))
                {
                if (!DeleteFile(BitmapFilename))
                    {
                    // something went wrong, probably bitmap file read-only
                    return S_FALSE;
                    }
                }

            SWImage = new Graphics::TBitmap();
            assert(SWImage);

            SWImage->Width  = Width;
            SWImage->Height = Height;

            pDIBData = new char[dwDataSize];
            assert(pDIBData);

            hr = pPreview->Read(pDIBData, dwDataSize, &dwNumRead);
            assert(dwNumRead == dwDataSize);

            lpbihDIBData = (LPBITMAPINFOHEADER ) pDIBData;

            DWORD dwLen = lpbihDIBData->biSize + (1 << lpbihDIBData->biBitCount) * sizeof(RGBQUAD);

            NumLines = StretchDIBits(
                            SWImage->Canvas->Handle,
                            Offset, Offset, Width - Offset * 2, Height - Offset * 2,
                            0, 0, lpbihDIBData->biWidth, lpbihDIBData->biHeight,
                            (LPSTR ) lpbihDIBData + dwLen,   // address of bitmap bits
                            (LPBITMAPINFO ) lpbihDIBData,    // address of bitmap data
                            DIB_RGB_COLORS, SRCCOPY);
            assert(NumLines);

            SWImage->SaveToFile(BitmapFilename);

            delete SWImage; 
            delete pDIBData;
            }
        }
    catch(Exception &e)
        {
        return E_UNEXPECTED;
        }
        
    // zero = no error
    *Error = 0;
    return S_OK;
}
//-----------------------------------------------------------------------

